<?php

namespace Drupal\datetime_range_ef\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\datetime_range\Plugin\Field\FieldFormatter\DateRangeDefaultFormatter;

/**
 * Plugin implementation of the 'carga_view' formatter.
 *
 * @FieldFormatter(
 *   id = "daterange_ef",
 *   label = @Translation("Date difference"),
 *   field_types = {
 *     "daterange"
 *   }
 * )
 */
class DateDiffFormatter extends DateRangeDefaultFormatter {

  /**
  * {@inheritdoc}
  */
  public static function defaultSettings() {
    return [
      'output_format' => 'full',
      'difference_separator' => '+',
    ] + parent::defaultSettings();
  }

  /**
  * Helper function to get the formatter settings options.
  *
  * @return array
  *   The formatter settings options.
  */
  protected function formatOptions() {
    return [
      'days' => $this->t("Difference in days"),
      'full' => $this->t("Difference in Years, months and days"),
      'dateDays' => $this->t("Start date +days"),
    ];
  }



  /**
   * Returns the output format, set or default one.
   *
   * @return string
   *   The output format string.
   */
  protected function getOutputFormat() {
    return in_array($this->getSetting('output_format'), array_keys($this->formatOptions())) ? $this->getSetting('output_format') : self::defaultSettings()['output_format'];
  }

  /**
  * {@inheritdoc}
  */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['output_format'] = [
      '#title' => $this->t('Style for the result'),
      '#type' => 'select',
      '#default_value' => $this->getOutputFormat(),
      '#options' => $this->formatOptions(),
      '#description' => $this->t('Style to use for the result'),
      '#required' => TRUE,
    ];
    unset($form['separator']);
    $form['difference_separator'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Separator from date'),
      '#description' => $this->t('The string to separate the start and the difference'),
      '#default_value' => $this->getSetting('difference_separator'),
      '#states' => array (
        'visible' => array(
          ':input[name="output_format"]' => array(
            'value' => 'dateDays',
            'selected' => 'selected',
          ),
        ),
      ),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $time_zone = drupal_get_user_timezone();
    $time_zone = new \DateTimeZone($time_zone);

    foreach ($items as $delta => $item) {
      if (!empty($item->start_date) && !empty($item->end_date)) {
        /** @var \Drupal\Core\Datetime\DrupalDateTime $start_date */
        $start_date = $item->start_date;
        /** @var \Drupal\Core\Datetime\DrupalDateTime $end_date */
        $end_date = $item->end_date;

        if ($start_date->getTimestamp() !== $end_date->getTimestamp()) {
          $date_diff = $this->calculateDiff($start_date, $end_date);

          $elements[$delta] = [
            '#type' => 'markup',
            '#markup' => $date_diff,
            '#cache' => ['contexts' => ['timezone',],],
          ];
        }
        else {
          $elements[$delta] = [
            '#type' => 'markup',
            '#markup' => '1' . t('day'),
            '#cache' => [
              'contexts' => [
                'timezone',
              ],
            ],
          ];
        }
      }
    }

    return $elements;
  }

  private function calculateDiff($start_date, $end_date) {
    $difference_separator = $this->getSetting('difference_separator');
    $output_format = $this->getSetting('output_format');
    //$date_format = \Drupal::entityTypeManager()->getStorage('date_format')->load($format_type);
    $date_diff = $end_date->getTimestamp() - $start_date->getTimestamp();
    $day_diff = floor($date_diff / (60 * 60 * 24)) + 1;
    $day_rest = $day_diff;
    if ($output_format === 'dateDays') {
      return $markup = $this->buildDate($start_date)['#markup'] . ' ' . $difference_separator . ' ' . $day_diff . ' ' . t('days');
    }
    if ($output_format === 'days') {
      return $markup = $day_diff . ' ' . t('days');
    }
    if ($output_format === 'full') {
      if ($day_diff >= 365 ) {
        $year_diff = floor($day_diff/365);
        if ($year_diff === 1) {
          $year_part = '1' . ' ' . t('year');
        }
        else {
          $year_part = $year_diff . ' ' . t('years');
        }
        $day_rest =  $day_rest - ($year_diff*365);
        if ($day_rest === 0) {
          return $markup = $year_part . ' (' . $day_diff . ' ' . t('days') . ')';
        }
        else {
          $year_part = $year_part . ', ';
        }
      }
      if ($day_rest >= 30) {
        $month_diff = floor($day_rest/30);
        if ($month_diff === 1) {
          $month_part = '1' . ' ' . t('month');
        }
        else {
          $month_part = $month_diff . ' ' . t('months');
        }
        $day_rest =  $day_rest - ($month_diff*30);
        if ($day_rest === 0) {
          return $markup = $year_part . $month_part . ' (' . $day_diff . ' ' . t('days') . ')';
        }
        else {
          $month_part = $month_part . ', ';
        }
      }
      if ($day_rest === 1) {
        return $markup = $year_part . $month_part . '1' . t('day') . ' (' . $day_diff . ' ' . t('days') . ')' ;
      }
      else {
        $markup = $year_part . $month_part . $day_rest . ' ' .t('days');
        if (!is_null($year_part) || !is_null($month_part)) {
          $markup = $markup . ' (' . $day_diff . ' ' . t('days') . ')' ;
        }
        return $markup;
      }

    }
  }
}
